@@ -21,6 +21,7 @@ import enum
 import math
 import datetime as dt
 import operator
+from uuid import UUID
 from decimal import Context, Decimal, localcontext
 from inspect import isclass, isfunction
 from fractions import Fraction
@@ -36,12 +37,30 @@ from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
 from hypothesis.internal.floats import next_up, next_down, is_negative, \
     float_to_int, int_to_float, count_between_floats
 from hypothesis.internal.charmap import as_general_categories
+from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
 from hypothesis.internal.validation import check_type, try_convert, \
     check_strategy, check_valid_size, check_valid_bound, \
-    check_valid_sizes, check_valid_integer, check_valid_interval
+    check_valid_sizes, check_valid_integer, check_valid_interval, \
+    check_valid_magnitude
+
+if False:
+    from random import Random  # noqa
+    from typing import Any, Dict, Union, Sequence, Callable, Pattern  # noqa
+    from typing import TypeVar, Tuple, List, Set, FrozenSet, overload  # noqa
+    from typing import Type, Mapping, Text, AnyStr, Optional  # noqa
+
+    from hypothesis.utils.conventions import InferType  # noqa
+    from hypothesis.searchstrategy.strategies import T, Ex  # noqa
+    K, V = TypeVar['K'], TypeVar['V']
+    # See https://github.com/python/mypy/issues/3186 - numbers.Real is wrong!
+    Real = Union[int, float, Fraction, Decimal]
+    ExtendFunc = Callable[[SearchStrategy[Union[T, Ex]]], SearchStrategy[T]]
+else:
+    def overload(f):
+        return f
 
 __all__ = [
     'nothing',
@@ -93,6 +112,7 @@ STRATEGY_CACHE = LRUReusedCache(1024)
 
 
 def cacheable(fn):
+    # type: (T) -> T
     @proxies(fn)
     def cached_strategy(*args, **kwargs):
         kwargs_cache_key = set()
@@ -118,6 +138,7 @@ def cacheable(fn):
 
 
 def base_defines_strategy(force_reusable):
+    # type: (bool) -> Callable[[T], T]
     """Returns a decorator for strategy functions.
 
     If force_reusable is True, the generated values are assumed to be
@@ -175,6 +196,7 @@ NOTHING = Nothing()
 
 @cacheable
 def nothing():
+    # type: () -> SearchStrategy
     """This strategy never successfully draws a value and will always reject on
     an attempt to draw.
 
@@ -184,6 +206,7 @@ def nothing():
 
 
 def just(value):
+    # type: (T) -> SearchStrategy[T]
     """Return a strategy which only generates ``value``.
 
     Note: ``value`` is not copied. Be wary of using mutable values.
@@ -201,6 +224,7 @@ def just(value):
 
 @defines_strategy_with_reusable_values
 def none():
+    # type: () -> SearchStrategy[None]
     """Return a strategy which only generates None.
 
     Examples from this strategy do not shrink (because there is only
@@ -209,12 +233,15 @@ def none():
     return just(None)
 
 
-def one_of(*args):
+def one_of(
+    *args  # type: Union[SearchStrategy[Ex], Sequence[SearchStrategy[Ex]]]
+):
+    # type: (...) -> SearchStrategy[Ex]
     """Return a strategy which generates values from any of the argument
     strategies.
 
     This may be called with one iterable argument instead of multiple
-    strategy arguments. In which case one_of(x) and one_of(\*x) are
+    strategy arguments. In which case ``one_of(x)`` and ``one_of(*x)`` are
     equivalent.
 
     Examples from this strategy will generally shrink to ones that come from
@@ -240,6 +267,7 @@ def one_of(*args):
 @cacheable
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
+    # type: (Real, Real) -> SearchStrategy[int]
     """Returns a strategy which generates integers (in Python 2 these may be
     ints or longs).
 
@@ -293,6 +321,7 @@ def integers(min_value=None, max_value=None):
 @cacheable
 @defines_strategy
 def booleans():
+    # type: () -> SearchStrategy[bool]
     """Returns a strategy which generates instances of bool.
 
     Examples from this strategy will shrink towards False (i.e.
@@ -430,23 +459,10 @@ def floats(
         return result
 
 
-@cacheable
-@defines_strategy_with_reusable_values
-def complex_numbers():
-    """Returns a strategy that generates complex numbers.
-
-    Examples from this strategy shrink by shrinking their component real
-    and imaginary parts.
-    """
-    from hypothesis.searchstrategy.numbers import ComplexStrategy
-    return ComplexStrategy(
-        tuples(floats(), floats())
-    )
-
-
 @cacheable
 @defines_strategy
 def tuples(*args):
+    # type: (*SearchStrategy) -> SearchStrategy[tuple]
     """Return a strategy which generates a tuple of the same length as args by
     generating the value at index i from args[i].
 
@@ -462,6 +478,19 @@ def tuples(*args):
     return TupleStrategy(args)
 
 
+@overload
+def sampled_from(elements):
+    # type: (Sequence[T]) -> SearchStrategy[T]
+    pass  # pragma: no cover
+
+
+@overload
+def sampled_from(elements):
+    # type: (Type[enum.Enum]) -> SearchStrategy[Any]
+    # `SearchStrategy[Enum]` is unreliable due to mataclass issues.
+    pass  # pragma: no cover
+
+
 @defines_strategy
 def sampled_from(elements):
     """Returns a strategy which generates any value present in ``elements``.
@@ -494,23 +523,24 @@ def sampled_from(elements):
     return SampledFromStrategy(values)
 
 
-_AVERAGE_LIST_LENGTH = 5.0
-
-
 @cacheable
 @defines_strategy
 def lists(
-    elements=None, min_size=None, average_size=None, max_size=None,
-    unique_by=None, unique=False,
+    elements=None,  # type: SearchStrategy[Ex]
+    min_size=None,  # type: int
+    average_size=None,  # type: int
+    max_size=None,  # type: int
+    unique_by=None,  # type: Callable[..., Any]
+    unique=False,  # type: bool
 ):
+    # type: (...) -> SearchStrategy[List[Ex]]
     """Returns a list containing values drawn from elements with length in the
     interval [min_size, max_size] (no bounds in that direction if these are
     None). If max_size is 0 then elements may be None and only the empty list
     will be drawn.
 
-    average_size may be used as a size hint to roughly control the size
-    of the list but it may not be the actual average of sizes you get, due
-    to a variety of factors.
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     If unique is True (or something that evaluates to True), we compare direct
     object equality, as if unique_by was `lambda x: x`. This comparison only
@@ -538,9 +568,10 @@ def lists(
                 'average_size=%r, max_size=%r) without elements.'
                 % (min_size, average_size, max_size)
             )
-        else:
-            return builds(list)
-    check_strategy(elements)
+        return builds(list)
+    if max_size == 0:
+        return builds(list)
+    check_strategy(elements, 'elements')
     if unique:
         if unique_by is not None:
             raise InvalidArgument((
@@ -551,40 +582,27 @@ def lists(
             def unique_by(x):
                 return x
 
-    if min_size is None:
-        min_size = 0
-
-    if max_size is None:
-        max_size = float('inf')
-
-    if average_size is None:
-        average_size = max(
-            _AVERAGE_LIST_LENGTH,
-            min_size * 2
-        )
-        if max_size < float('inf'):
-            average_size = min(average_size, 0.5 * (min_size + max_size))
-
     from hypothesis.searchstrategy.collections import ListStrategy, \
         UniqueListStrategy
     if unique_by is not None:
         return UniqueListStrategy(
             elements=elements,
-            average_size=average_size,
             max_size=max_size,
             min_size=min_size,
             key=unique_by
         )
-    else:
-        return ListStrategy(
-            elements, average_size=average_size,
-            min_size=min_size, max_size=max_size,
-        )
+    return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
 
 @cacheable
 @defines_strategy
-def sets(elements=None, min_size=None, average_size=None, max_size=None):
+def sets(
+    elements=None,  # type: SearchStrategy[Ex]
+    min_size=None,   # type: int
+    average_size=None,  # type: int
+    max_size=None,  # type: int
+):
+    # type: (...) -> SearchStrategy[Set[Ex]]
     """This has the same behaviour as lists, but returns sets instead.
 
     Note that Hypothesis cannot tell if values are drawn from elements
@@ -608,7 +626,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 
 @cacheable
 @defines_strategy
-def frozensets(elements=None, min_size=None, average_size=None, max_size=None):
+def frozensets(
+    elements=None,  # type: SearchStrategy[Ex]
+    min_size=None,   # type: int
+    average_size=None,  # type: int
+    max_size=None,  # type: int
+):
+    # type: (...) -> SearchStrategy[FrozenSet[Ex]]
     """This is identical to the sets function but instead returns
     frozensets."""
     if elements is None:
@@ -663,7 +687,10 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
 
 
 @defines_strategy
-def fixed_dictionaries(mapping):
+def fixed_dictionaries(
+    mapping  # type: Mapping[T, SearchStrategy[Ex]]
+):
+    # type: (...) -> SearchStrategy[Dict[T, Ex]]
     """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. mapping must be a dict subclass.
 
@@ -685,9 +712,14 @@ def fixed_dictionaries(mapping):
 @cacheable
 @defines_strategy
 def dictionaries(
-    keys, values, dict_class=dict,
-    min_size=None, average_size=None, max_size=None
+    keys,  # type: SearchStrategy[Ex]
+    values,  # type: SearchStrategy[T]
+    dict_class=dict,  # type: Type[Mapping]
+    min_size=None,  # type: int
+    average_size=None,  # type: int
+    max_size=None,  # type: int
 ):
+    # type: (...) -> SearchStrategy[Mapping[Ex, T]]
     """Generates dictionaries of type dict_class with keys drawn from the keys
     argument and values drawn from the values argument.
 
@@ -704,7 +736,7 @@ def dictionaries(
 
     return lists(
         tuples(keys, values),
-        min_size=min_size, average_size=average_size, max_size=max_size,
+        min_size=min_size, max_size=max_size,
         unique_by=lambda x: x[0]
     ).map(dict_class)
 
@@ -735,9 +767,15 @@ def streaming(elements):
 
 @cacheable
 @defines_strategy_with_reusable_values
-def characters(whitelist_categories=None, blacklist_categories=None,
-               blacklist_characters=None, min_codepoint=None,
-               max_codepoint=None, whitelist_characters=None):
+def characters(
+    whitelist_categories=None,  # type: Sequence[Text]
+    blacklist_categories=None,  # type: Sequence[Text]
+    blacklist_characters=None,  # type: Sequence[Text]
+    min_codepoint=None,  # type: int
+    max_codepoint=None,  # type: int
+    whitelist_characters=None,  # type: Sequence[Text]
+):
+    # type: (...) -> SearchStrategy[Text]
     """Generates unicode text type (unicode on python 2, str on python 3)
     characters following specified filtering rules.
 
@@ -825,9 +863,12 @@ def characters(whitelist_categories=None, blacklist_categories=None,
 @cacheable
 @defines_strategy_with_reusable_values
 def text(
-    alphabet=None,
-    min_size=None, average_size=None, max_size=None
+    alphabet=None,  # type: Union[Text, SearchStrategy[Text]]
+    min_size=None,   # type: int
+    average_size=None,   # type: int
+    max_size=None  # type: int
 ):
+    # type: (...) -> SearchStrategy[Text]
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
     length one strings or a strategy generating such. If it is None it will
@@ -835,12 +876,16 @@ def text(
     characters). If it is an empty collection this will only generate empty
     strings.
 
-    min_size, max_size and average_size have the usual interpretations.
+    min_size and max_size have the usual interpretations.
+
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards shorter strings, and with the
     characters in the text shrinking as per the alphabet strategy.
     """
     from hypothesis.searchstrategy.strings import StringStrategy
+    check_valid_sizes(min_size, average_size, max_size)
     if alphabet is None:
         char_strategy = characters(blacklist_categories=('Cs',))
     elif not alphabet:
@@ -856,14 +901,14 @@ def text(
     else:
         char_strategy = sampled_from(list(map(text_type, alphabet)))
     return StringStrategy(lists(
-        char_strategy, average_size=average_size, min_size=min_size,
-        max_size=max_size
+        char_strategy, min_size=min_size, max_size=max_size
     ))
 
 
 @cacheable
 @defines_strategy
 def from_regex(regex):
+    # type: (Union[AnyStr, Pattern[AnyStr]]) -> SearchStrategy[AnyStr]
     """Generates strings that contain a match for the given regex (i.e. ones
     for which :func:`re.search` will return a non-None result).
 
@@ -898,10 +943,14 @@ def from_regex(regex):
 def binary(
     min_size=None, average_size=None, max_size=None
 ):
+    # type: (int, int, int) -> SearchStrategy[bytes]
     """Generates the appropriate binary type (str in python 2, bytes in python
     3).
 
-    min_size, average_size and max_size have the usual interpretations.
+    min_size and max_size have the usual interpretations.
+
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards smaller strings and lower byte
     values.
@@ -914,7 +963,7 @@ def binary(
     return BinaryStringStrategy(
         lists(
             integers(min_value=0, max_value=255),
-            average_size=average_size, min_size=min_size, max_size=max_size
+            min_size=min_size, max_size=max_size
         )
     )
 
@@ -922,6 +971,7 @@ def binary(
 @cacheable
 @defines_strategy
 def randoms():
+    # type: () -> SearchStrategy[Random]
     """Generates instances of Random (actually a Hypothesis specific
     RandomWithSeed class which displays what it was initially seeded with)
 
@@ -972,7 +1022,11 @@ def random_module():
 
 @cacheable
 @defines_strategy
-def builds(*callable_and_args, **kwargs):
+def builds(
+    *callable_and_args,  # type: Any
+    **kwargs  # type: Union[SearchStrategy[Any], InferType]
+):
+    # type: (...) -> SearchStrategy[Any]
     """Generates values by drawing from ``args`` and ``kwargs`` and passing
     them to the callable (provided as the first positional argument) in the
     appropriate argument position.
@@ -996,7 +1050,7 @@ def builds(*callable_and_args, **kwargs):
                 'The first positional argument to builds() must be a callable '
                 'target to construct.')
     elif 'target' in kwargs and callable(kwargs['target']):
-        args = []
+        args = ()
         note_deprecation(
             'Specifying the target as a keyword argument to builds() is '
             'deprecated. Provide it as the first positional argument instead.')
@@ -1020,12 +1074,15 @@ def builds(*callable_and_args, **kwargs):
     required = required_args(target, args, kwargs)
     for ms in set(hints) & (required or set()):
         kwargs[ms] = from_type(hints[ms])
-    return tuples(tuples(*args), fixed_dictionaries(kwargs)).map(
+    # Mypy doesn't realise that `infer` is gone from kwargs now
+    kwarg_strat = fixed_dictionaries(kwargs)  # type: ignore
+    return tuples(tuples(*args), kwarg_strat).map(
         lambda value: target(*value[0], **value[1])
     )
 
 
 def _defer_from_type(func):
+    # type: (T) -> T
     """Decorator to make from_type lazy to support recursive definitions."""
     @proxies(func)
     def inner(*args, **kwargs):
@@ -1036,6 +1093,7 @@ def _defer_from_type(func):
 @cacheable
 @_defer_from_type
 def from_type(thing):
+    # type: (Type[Ex]) -> SearchStrategy[Ex]
     """Looks up the appropriate search strategy for the given type.
 
     ``from_type`` is used internally to fill in missing arguments to
@@ -1064,6 +1122,7 @@ def from_type(thing):
     from hypothesis.searchstrategy import types
     try:
         import typing
+        fr = typing._ForwardRef  # type: ignore
         if not isinstance(thing, type):
             # At runtime, `typing.NewType` returns an identity function rather
             # than an actual type, but we can check that for a possible match
@@ -1080,7 +1139,7 @@ def from_type(thing):
                 return one_of([from_type(t) for t in args])
         # We can't resolve forward references, and under Python 3.5 (only)
         # a forward reference is an instance of type.  Hence, explicit check:
-        elif type(thing) == typing._ForwardRef:  # pragma: no cover
+        elif type(thing) == fr:  # pragma: no cover
             raise ResolutionFailed(
                 'thing=%s cannot be resolved.  Upgrading to python>=3.6 may '
                 'fix this problem via improvements to the typing module.'
@@ -1109,7 +1168,7 @@ def from_type(thing):
     # subclass and instance checks.
     try:
         import typing
-        if isinstance(thing, typing.TypingMeta):
+        if isinstance(thing, typing.TypingMeta):  # type: ignore
             return types.from_typing_type(thing)
     except ImportError:  # pragma: no cover
         pass
@@ -1120,8 +1179,9 @@ def from_type(thing):
     strategies = [
         v if isinstance(v, SearchStrategy) else v(thing)
         for k, v in types._global_type_lookup.items()
-        if issubclass(k, thing) and
-        sum(types.try_issubclass(k, T) for T in types._global_type_lookup) == 1
+        if issubclass(k, thing) and sum(
+            types.try_issubclass(k, typ) for typ in types._global_type_lookup
+        ) == 1
     ]
     empty = ', '.join(repr(s) for s in strategies if s.is_empty)
     if empty:
@@ -1153,7 +1213,12 @@ def from_type(thing):
 
 @cacheable
 @defines_strategy_with_reusable_values
-def fractions(min_value=None, max_value=None, max_denominator=None):
+def fractions(
+    min_value=None,  # type: Union[Real, AnyStr]
+    max_value=None,  # type: Union[Real, AnyStr]
+    max_denominator=None,  # type: int
+):
+    # type: (...) -> SearchStrategy[Fraction]
     """Returns a strategy which generates Fractions.
 
     If min_value is not None then all generated values are no less than
@@ -1171,6 +1236,10 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
     min_value = try_convert(Fraction, min_value, 'min_value')
     max_value = try_convert(Fraction, max_value, 'max_value')
 
+    if (min_value is not None and not isinstance(min_value, Fraction) or
+            max_value is not None and not isinstance(max_value, Fraction)):
+        assert False, 'Unreachable for Mypy'  # pragma: no cover
+
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
     check_valid_integer(max_denominator)
 
@@ -1180,9 +1249,11 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
                 'max_denominator=%r must be >= 1' % max_denominator)
 
         def fraction_bounds(value):
+            # type: (Fraction) -> Tuple[Fraction, Fraction]
             """Find the best lower and upper approximation for value."""
             # Adapted from CPython's Fraction.limit_denominator here:
             # https://github.com/python/cpython/blob/3.6/Lib/fractions.py#L219
+            assert max_denominator is not None
             if value is None or value.denominator <= max_denominator:
                 return value, value
             p0, q0, p1, q1 = 0, 1, 1, 0
@@ -1201,10 +1272,13 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
 
         # Take the high approximation for min_value and low for max_value
         bounds = (max_denominator, min_value, max_value)
-        _, min_value = fraction_bounds(min_value)
-        max_value, _ = fraction_bounds(max_value)
+        if min_value is not None:
+            _, min_value = fraction_bounds(min_value)
+        if max_value is not None:
+            max_value, _ = fraction_bounds(max_value)
 
-        if None not in (min_value, max_value) and min_value > max_value:
+        if min_value is not None and max_value is not None and \
+                min_value > max_value:
             raise InvalidArgument(
                 'There are no fractions with a denominator <= %r between '
                 'min_value=%r and max_value=%r' % bounds)
@@ -1249,7 +1323,12 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
         lambda f: f.limit_denominator(max_denominator))
 
 
-def _as_finite_decimal(value, name, allow_infinity):
+def _as_finite_decimal(
+    value,  # type: Union[Real, AnyStr, None]
+    name,  # type: str
+    allow_infinity,  # type: Optional[bool]
+):
+    # type: (...) -> Optional[Decimal]
     """Convert decimal bounds to decimals, carefully."""
     assert name in ('min_value', 'max_value')
     if value is None:
@@ -1257,6 +1336,7 @@ def _as_finite_decimal(value, name, allow_infinity):
     if not isinstance(value, Decimal):
         with localcontext(Context()):  # ensure that default traps are enabled
             value = try_convert(Decimal, value, name)
+    assert isinstance(value, Decimal)
     if value.is_finite():
         return value
     if value.is_infinite() and (value < 0 if 'min' in name else value > 0):
@@ -1270,8 +1350,14 @@ def _as_finite_decimal(value, name, allow_infinity):
 
 @cacheable
 @defines_strategy_with_reusable_values
-def decimals(min_value=None, max_value=None,
-             allow_nan=None, allow_infinity=None, places=None):
+def decimals(
+    min_value=None,  # type: Union[Real, AnyStr]
+    max_value=None,  # type: Union[Real, AnyStr]
+    allow_nan=None,  # type: bool
+    allow_infinity=None,  # type: bool
+    places=None,  # type: int
+):
+    # type: (...) -> SearchStrategy[Decimal]
     """Generates instances of :class:`decimals.Decimal`, which may be:
 
     - A finite rational number, between ``min_value`` and ``max_value``.
@@ -1322,7 +1408,7 @@ def decimals(min_value=None, max_value=None,
             min_num = ceil(ctx(min_value).divide(min_value, factor))
         if max_value is not None:
             max_num = floor(ctx(max_value).divide(max_value, factor))
-        if None not in (min_num, max_num) and min_num > max_num:
+        if min_num is not None and max_num is not None and min_num > max_num:
             raise InvalidArgument(
                 'There are no decimals with %d places between min_value=%r '
                 'and max_value=%r ' % (places, min_value, max_value))
@@ -1337,7 +1423,7 @@ def decimals(min_value=None, max_value=None,
 
         strat = fractions(min_value, max_value).map(fraction_to_decimal)
     # Compose with sampled_from for infinities and NaNs as appropriate
-    special = []
+    special = []  # type: List[Decimal]
     if allow_nan or (allow_nan is None and (None in (min_value, max_value))):
         special.extend(map(Decimal, ('NaN', '-NaN', 'sNaN', '-sNaN')))
     if allow_infinity or (allow_infinity is max_value is None):
@@ -1347,7 +1433,12 @@ def decimals(min_value=None, max_value=None,
     return strat | sampled_from(special)
 
 
-def recursive(base, extend, max_leaves=100):
+def recursive(
+    base,  # type: SearchStrategy[Ex]
+    extend,  # type: ExtendFunc
+    max_leaves=100,  # type: int
+):
+    # type: (...) -> SearchStrategy[Union[T, Ex]]
     """base: A strategy to start from.
 
     extend: A function which takes a strategy and returns a new strategy.
@@ -1376,6 +1467,7 @@ def recursive(base, extend, max_leaves=100):
 
 @defines_strategy
 def permutations(values):
+    # type: (Sequence[T]) -> SearchStrategy[List[T]]
     """Return a strategy which returns permutations of the collection
     ``values``.
 
@@ -1408,16 +1500,20 @@ def permutations(values):
     max_datetime='max_value',
 )
 def datetimes(
-    min_value=dt.datetime.min, max_value=dt.datetime.max,
-    timezones=none(),
-    min_datetime=None, max_datetime=None,
+    min_value=dt.datetime.min,  # type: dt.datetime
+    max_value=dt.datetime.max,  # type: dt.datetime
+    timezones=none(),  # type: SearchStrategy[Optional[dt.tzinfo]]
+    min_datetime=None,  # type: dt.datetime
+    max_datetime=None,  # type: dt.datetime
 ):
+    # type: (...) -> SearchStrategy[dt.datetime]
     """A strategy for generating datetimes, which may be timezone-aware.
 
     This strategy works by drawing a naive datetime between ``min_datetime``
     and ``max_datetime``, which must both be naive (have no timezone).
 
-    ``timezones`` must be a strategy that generates tzinfo objects (or None,
+    ``timezones`` must be a strategy that generates
+    :class:`~python:datetime.tzinfo` objects (or None,
     which is valid for naive datetimes).  A value drawn from this strategy
     will be added to a naive datetime, and the resulting tz-aware datetime
     returned.
@@ -1428,9 +1524,9 @@ def datetimes(
         adjustments, etc.  This is intentional, as malformed timestamps are a
         common source of bugs.
 
-    :py:func:`hypothesis.extra.timezones` requires the ``pytz`` package, but
-    provides all timezones in the Olsen database.  If you also want to allow
-    naive datetimes, combine strategies like ``none() | timezones()``.
+    :py:func:`hypothesis.extra.pytz.timezones` requires the :pypi:`pytz`
+    package, but provides all timezones in the Olsen database.  If you want to
+    allow naive datetimes, combine strategies like ``none() | timezones()``.
 
     Alternatively, you can create a list of the timezones you wish to allow
     (e.g. from the standard library, ``datetutil``, or ``pytz``) and use
@@ -1476,6 +1572,7 @@ def dates(
     min_value=dt.date.min, max_value=dt.date.max,
     min_date=None, max_date=None,
 ):
+    # type: (dt.date, dt.date, dt.date, dt.date) -> SearchStrategy[dt.date]
     """A strategy for dates between ``min_date`` and ``max_date``.
 
     Examples from this strategy shrink towards January 1st 2000.
@@ -1496,9 +1593,13 @@ def dates(
     max_time='max_value',
 )
 def times(
-    min_value=dt.time.min, max_value=dt.time.max, timezones=none(),
-    min_time=None, max_time=None,
+    min_value=dt.time.min,  # type: dt.time
+    max_value=dt.time.max,  # type: dt.time
+    timezones=none(),  # type: SearchStrategy
+    min_time=None,  # type: dt.time
+    max_time=None,  # type: dt.time
 ):
+    # type: (...) -> SearchStrategy[dt.time]
     """A strategy for times between ``min_time`` and ``max_time``.
 
     The ``timezones`` argument is handled as for :py:func:`datetimes`.
@@ -1525,9 +1626,12 @@ def times(
     max_delta='max_value',
 )
 def timedeltas(
-    min_value=dt.timedelta.min, max_value=dt.timedelta.max,
-    min_delta=None, max_delta=None
+    min_value=dt.timedelta.min,  # type: dt.timedelta
+    max_value=dt.timedelta.max,  # type: dt.timedelta
+    min_delta=None,  # type: dt.timedelta
+    max_delta=None,  # type: dt.timedelta
 ):
+    # type: (...) -> SearchStrategy[dt.timedelta]
     """A strategy for timedeltas between ``min_value`` and ``max_value``.
 
     Examples from this strategy shrink towards zero.
@@ -1544,6 +1648,7 @@ def timedeltas(
 
 @cacheable
 def composite(f):
+    # type: (Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]
     """Defines a strategy that is built out of potentially arbitrarily many
     other strategies.
 
@@ -1599,7 +1704,93 @@ def composite(f):
     return accept
 
 
+@defines_strategy_with_reusable_values
+@cacheable
+def complex_numbers(min_magnitude=0, max_magnitude=None,
+                    allow_infinity=None, allow_nan=None):
+    # type: (Optional[Real], Real, bool, bool) -> SearchStrategy[complex]
+    """Returns a strategy that generates complex numbers.
+
+    This strategy draws complex numbers with constrained magnitudes.
+    The ``min_magnitude`` and ``max_magnitude`` parameters should be
+    non-negative :class:`~python:numbers.Real` numbers; values
+    of ``None`` correspond to zero and infinite values respectively.
+
+    If ``min_magnitude`` is positive or ``max_magnitude`` is finite, it
+    is an error to enable ``allow_nan``.  If ``max_magnitude`` is finite,
+    it is an error to enable ``allow_infinity``.
+
+    The magnitude contraints are respected up to a relative error
+    of (around) floating-point epsilon, due to implementation via
+    the system ``sqrt`` function.
+
+    Examples from this strategy shrink by shrinking their real and
+    imaginary parts, as :func:`~hypothesis.strategies.floats`.
+
+    If you need to generate complex numbers with particular real and
+    imaginary parts or relationships between parts, consider using
+    `builds(complex, ...) <hypothesis.strategies.builds>` or
+    `@composite <hypothesis.strategies.composite>` respectively.
+    """
+    check_valid_magnitude(min_magnitude, 'min_magnitude')
+    check_valid_magnitude(max_magnitude, 'max_magnitude')
+    check_valid_interval(min_magnitude, max_magnitude,
+                         'min_magnitude', 'max_magnitude')
+    if max_magnitude == float('inf'):
+        max_magnitude = None
+    if min_magnitude == 0:
+        min_magnitude = None
+
+    if allow_infinity is None:
+        allow_infinity = bool(max_magnitude is None)
+    elif allow_infinity and max_magnitude is not None:
+        raise InvalidArgument(
+            'Cannot have allow_infinity=%r with max_magnitude=%r' %
+            (allow_infinity, max_magnitude)
+        )
+    if allow_nan is None:
+        allow_nan = bool(min_magnitude is None and max_magnitude is None)
+    elif allow_nan and not (min_magnitude is None and max_magnitude is None):
+        raise InvalidArgument(
+            'Cannot have allow_nan=%r, min_magnitude=%r max_magnitude=%r' %
+            (allow_nan, min_magnitude, max_magnitude)
+        )
+    allow_kw = dict(allow_nan=allow_nan, allow_infinity=allow_infinity)
+
+    if min_magnitude is None and max_magnitude is None:
+        # In this simple but common case, there are no constraints on the
+        # magnitude and therefore no relationship between the real and
+        # imaginary parts.
+        return builds(complex, floats(**allow_kw), floats(**allow_kw))
+
+    @composite
+    def constrained_complex(draw):
+        # Draw the imaginary part, and determine the maximum real part given
+        # this and the max_magnitude
+        if max_magnitude is None:
+            zi = draw(floats(**allow_kw))
+            rmax = float('inf')
+        else:
+            zi = draw(floats(-max_magnitude, max_magnitude, **allow_kw))
+            rmax = cathetus(max_magnitude, zi)
+        # Draw the real part from the allowed range given the imaginary part
+        if min_magnitude is None or math.fabs(zi) >= min_magnitude:
+            zr = draw(floats(-rmax, rmax, **allow_kw))
+        else:
+            zr = draw(floats(cathetus(min_magnitude, zi), rmax, **allow_kw))
+        # Order of conditions carefully tuned so that for a given pair of
+        # magnitude arguments, we always either draw or do not draw the bool
+        # (crucial for good shrinking behaviour) but only invert when needed.
+        if min_magnitude is not None and draw(booleans()) and \
+                math.fabs(zi) <= min_magnitude:
+            zr = -zr
+        return complex(zr, zi)
+
+    return constrained_complex()
+
+
 def shared(base, key=None):
+    # type: (SearchStrategy[Ex], Any) -> SearchStrategy[Ex]
     """Returns a strategy that draws a single shared value per run, drawn from
     base. Any two shared instances with the same key will share the same value,
     otherwise the identity of this strategy will be used. That is:
@@ -1677,6 +1868,7 @@ def choices():
 @cacheable
 @defines_strategy_with_reusable_values
 def uuids(version=None):
+    # type: (int) -> SearchStrategy[UUID]
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
     If the optional version argument is given, value is passed through
@@ -1688,7 +1880,6 @@ def uuids(version=None):
 
     Examples from this strategy don't have any meaningful shrink order.
     """
-    from uuid import UUID
     if version not in (None, 1, 2, 3, 4, 5):
         raise InvalidArgument((
             'version=%r, but version must be in (None, 1, 2, 3, 4, 5) '
@@ -1729,6 +1920,7 @@ def runner(default=not_set):
 
 @cacheable
 def data():
+    # type: () -> SearchStrategy[Any]
     """This isn't really a normal strategy, but instead gives you an object
     which can be used to draw data interactively from other strategies.
 
@@ -1796,6 +1988,7 @@ def data():
 
 
 def register_type_strategy(custom_type, strategy):
+    # type: (type, Union[type, Callable[[type], SearchStrategy]]) -> None
     """Add an entry to the global type-to-strategy lookup.
 
     This lookup is used in :func:`~hypothesis.strategies.builds` and
@@ -1820,11 +2013,12 @@ def register_type_strategy(custom_type, strategy):
     elif isinstance(strategy, SearchStrategy) and strategy.is_empty:
         raise InvalidArgument('strategy=%r must not be empty')
     types._global_type_lookup[custom_type] = strategy
-    from_type.__clear_cache()
+    from_type.__clear_cache()  # type: ignore
 
 
 @cacheable
 def deferred(definition):
+    # type: (Callable[[], SearchStrategy[Ex]]) -> SearchStrategy[Ex]
     """A deferred strategy allows you to write a strategy that references other
     strategies that have not yet been defined. This allows for the easy
     definition of recursive and mutually recursive strategies.
